"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cheerio_1 = require("cheerio");
const koishi_1 = require("koishi");
const utils_1 = require("./utils");
const baseURL = 'https://ascii2d.net';
const logger = new koishi_1.Logger('search');
async function default_1(http, url, session, config) {
    try {
        const tasks = [];
        const colorHTML = await http.get(`${baseURL}/search/url/${encodeURIComponent(url)}`, {
            headers: {
                'User-Agent': 'PostmanRuntime/7.29.0',
            },
        });
        tasks.push(session.send('ascii2d 色合检索\n' + getDetail(colorHTML, config.output)));
        try {
            const bovwURL = getTokuchouUrl(colorHTML);
            const bovwHTML = await http.get(bovwURL, {
                headers: {
                    'User-Agent': 'PostmanRuntime/7.29.0',
                    'Content-Type': 'multipart/form-data',
                },
            });
            tasks.push(session.send('ascii2d 特征检索\n' + getDetail(bovwHTML, config.output)));
        }
        catch (err) {
            logger.warn(`[error] ascii2d bovw ${err}`);
        }
        await Promise.all(tasks);
    }
    catch (err) {
        logger.warn(`[error] ascii2d color ${err}`);
        logger.warn(err);
        return '访问失败。';
    }
}
exports.default = default_1;
function getDetail(html, config) {
    const $ = (0, cheerio_1.load)(html, { decodeEntities: false });
    const $box = $($('.item-box')[1]);
    if ($box.length === 0) {
        logger.warn('[error] ascii2d bovw cannot find images in web page');
        return '没有找到相似图片。';
    }
    const thumbnail = baseURL + $box.find('.image-box img').attr('src');
    const $link = $box.find('.detail-box a');
    const $title = $($link[0]);
    const $author = $($link[1]);
    return (0, utils_1.getShareText)({
        imageUrl: $title.attr('href'),
        title: $author
            ? `「${$title.html()}」/「${$author.html()}」`
            : $title.html(),
        thumbnail,
        authorUrl: $author.attr('href'),
    }, config);
}
function getTokuchouUrl(html) {
    const $ = (0, cheerio_1.load)(html, { decodeEntities: false });
    return `${baseURL}/search/bovw/${$($('.hash')[0]).text().trim()}`;
}
