"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.apply = exports.inject = exports.name = exports.Config = void 0;
const koishi_1 = require("koishi");
const ascii2d_1 = __importDefault(require("./ascii2d"));
const saucenao_1 = __importDefault(require("./saucenao"));
const iqdb_1 = __importDefault(require("./iqdb"));
const utils_1 = require("./utils");
Object.defineProperty(exports, "Config", { enumerable: true, get: function () { return utils_1.Config; } });
exports.name = 'image-search';
exports.inject = ['http'];
async function mixedSearch(http, url, session, config) {
    return await (0, saucenao_1.default)(http, url, session, config, true) && (0, ascii2d_1.default)(http, url, session, config);
}
function apply(ctx, config = {}) {
    let index = 0;
    const http = ctx.http.extend(config);
    const keys = (0, koishi_1.makeArray)(config.saucenaoApiKey);
    ctx.on('saucenao/get-key', () => {
        const result = keys[index];
        index = (index + 1) % keys.length;
        return result;
    });
    ctx.on('saucenao/drop-key', (key) => {
        if (keys.indexOf(key) < 0)
            return;
        if (index === 0) {
            keys.pop();
        }
        else {
            keys.splice(--index, 1);
        }
        return '令牌失效导致访问失败，请联系机器人作者。';
    });
    ctx.command('search [image:text]', '搜图片')
        .shortcut('搜图', { fuzzy: true })
        .action(search(mixedSearch));
    ctx.command('search/saucenao [image:text]', '使用 saucenao 搜图')
        .action(search(saucenao_1.default));
    ctx.command('search/ascii2d [image:text]', '使用 ascii2d 搜图')
        .action(search(ascii2d_1.default));
    ctx.command('search/iqdb [image:text]', '使用 iqdb 搜图')
        .action(search(iqdb_1.default));
    const pendings = new Set();
    async function searchUrl(session, url, callback) {
        const id = session.channelId;
        pendings.add(id);
        try {
            const result = await callback(http, url, session, config);
            if (typeof result === 'string')
                return result;
        }
        catch (error) {
            ctx.logger('search').warn(error);
            return '搜索失败。';
        }
        finally {
            pendings.delete(id);
        }
    }
    function getUrl(input) {
        const [el] = koishi_1.h.select(input || [], 'image, img');
        return el?.attrs.src || el?.attrs.url;
    }
    function search(callback) {
        return async ({ session }, image) => {
            const id = session.channelId;
            if (pendings.has(id))
                return '存在正在进行的查询，请稍后再试。';
            const url = getUrl(image);
            if (url) {
                pendings.add(id);
                return searchUrl(session, url, callback);
            }
            const dispose = session.middleware(({ content }, next) => {
                dispose();
                const url = getUrl(content);
                if (!url)
                    return next();
                return searchUrl(session, url, callback);
            });
            return '请发送图片。';
        };
    }
}
exports.apply = apply;
