"use strict";
/* eslint-disable camelcase */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nhentai_1 = __importDefault(require("./nhentai"));
const danbooru_1 = __importDefault(require("./danbooru"));
const konachan_1 = __importDefault(require("./konachan"));
const koishi_1 = require("koishi");
const utils_1 = require("./utils");
const logger = new koishi_1.Logger('search');
var Params;
(function (Params) {
    let Type;
    (function (Type) {
        Type[Type["html"] = 0] = "html";
        Type[Type["xml"] = 1] = "xml";
        Type[Type["json"] = 2] = "json";
    })(Type = Params.Type || (Params.Type = {}));
    let Dedupe;
    (function (Dedupe) {
        Dedupe[Dedupe["none"] = 0] = "none";
        Dedupe[Dedupe["consolidate"] = 1] = "consolidate";
        Dedupe[Dedupe["all"] = 2] = "all";
    })(Dedupe = Params.Dedupe || (Params.Dedupe = {}));
})(Params || (Params = {}));
async function search(http, url, session, config, mixed) {
    const { app } = session;
    const keys = new Set();
    for (let i = 0; i < config.maxTrials || 3; ++i) {
        const api_key = app.bail('saucenao/get-key');
        if (!api_key || keys.has(api_key)) {
            if (!mixed)
                await session.send('当前没有可用的 API 令牌，请联系机器人作者。');
            return;
        }
        keys.add(api_key);
        try {
            return await http.get('https://saucenao.com/search.php', {
                params: {
                    db: 999,
                    numres: 3,
                    api_key,
                    url,
                    output_type: Params.Type.json,
                },
            });
        }
        catch (err) {
            if (!err.response) {
                if (!(err instanceof Error) || !err.message.includes('ECONNRESET') && !err.message.includes('ECONNREFUSED')) {
                    logger.warn(`[error] saucenao:`, err);
                }
                await session.send('无法连接服务器。');
                return;
            }
            else if (err.response.status === 403) {
                const result = app.bail('saucenao/drop-key', api_key);
                if (result) {
                    await session.send(result);
                    return;
                }
            }
            else if (err.response.status !== 429) {
                logger.warn(`[error] saucenao:`, err.response.data);
                await session.send('由于未知原因搜索失败。');
                return;
            }
        }
    }
    await session.send('搜索次数已达单位时间上限，请稍候再试。');
}
async function saucenao(http, url, session, config, mixed) {
    const data = await search(http, url, session, config, mixed);
    if (!data)
        return;
    if (!data.results) {
        const message = data.header.message.toLowerCase();
        if (message.includes('you need an image')) {
            await session.send('没有传入图片 URL。');
            return;
        }
        else if (message.includes('supplied url is not usable')) {
            await session.send('无法使用传入的图片 URL。');
            return;
        }
        logger.warn(`[error] saucenao:`, data.header);
        await session.send('由于未知原因搜索失败：' + data.header.message);
        return;
    }
    if (!data.results.length) {
        await session.send('没有找到搜索结果。');
        return;
    }
    const { long_remaining, short_remaining } = data.header;
    const output = [];
    const { similarity } = data.results[0].header;
    const lowSimilarity = +similarity < (config.lowSimilarity ?? 40);
    const highSimilarity = +similarity > (config.highSimilarity ?? 60);
    if (lowSimilarity) {
        output.push(`相似度 (${similarity}%) 过低，这很可能不是你要找的图。`);
    }
    else if (!highSimilarity) {
        output.push(`相似度 (${similarity}%) 较低，这可能不是你要找的图。`);
        if (mixed)
            output[0] += '将自动使用 ascii2d 继续进行搜索。';
    }
    if (!lowSimilarity || !mixed)
        await handleResult(data.results[0], session, output, config.output);
    if (long_remaining < 20) {
        output.push(`注意：24h 内搜图次数仅剩 ${long_remaining} 次。`);
    }
    else if (short_remaining < 3) {
        output.push(`注意：30s 内搜图次数仅剩 ${short_remaining} 次。`);
    }
    await session.send(output.join('\n'));
    return !highSimilarity && mixed;
}
async function handleResult(result, session, output, config) {
    const { header, data } = result;
    const { thumbnail, similarity } = header;
    const { ext_urls, title, member_id, member_name, eng_name, jp_name } = data;
    let imageUrl;
    let source;
    if (ext_urls) {
        imageUrl = ext_urls[0];
        for (let i = 1; i < ext_urls.length; i++) {
            if ((0, utils_1.checkHost)(ext_urls[i], 'danbooru')) {
                imageUrl = ext_urls[i];
                break;
            }
        }
        if ((0, utils_1.checkHost)(imageUrl, 'danbooru')) {
            source = await (0, danbooru_1.default)(imageUrl, session).catch(logger.debug);
        }
        else if ((0, utils_1.checkHost)(imageUrl, 'konachan')) {
            source = await (0, konachan_1.default)(imageUrl, session).catch(logger.debug);
        }
    }
    if (jp_name || eng_name) {
        const bookName = (jp_name || eng_name).replace('(English)', '');
        try {
            const book = await (0, nhentai_1.default)(bookName);
            if (book) {
                imageUrl = `https://nhentai.net/g/${book.id}/`;
            }
            else {
                output.push('没有在 nhentai 找到对应的本子_(:3」∠)_');
            }
        }
        catch (error) {
            logger.debug(error);
        }
        output.push((0, utils_1.getShareText)({
            imageUrl,
            thumbnail,
            title: `(${similarity}%) ${bookName}`,
        }, config));
    }
    else {
        const displayTitle = member_name
            ? `「${title}」/「${member_name}」`
            : title || ((0, utils_1.checkHost)(imageUrl, 'anidb.net') ? 'AniDB' : '搜索结果');
        output.push((0, utils_1.getShareText)({
            imageUrl,
            thumbnail,
            title: `(${similarity}%) ${displayTitle}`,
            authorUrl: member_id && (0, utils_1.checkHost)(imageUrl, 'pixiv.net') && `https://www.pixiv.net/u/${member_id}`,
            source,
        }, config));
    }
}
(function (saucenao) {
    saucenao.Config = koishi_1.Schema.object({
        saucenaoApiKey: koishi_1.Schema.union([
            koishi_1.Schema.array(String),
            koishi_1.Schema.transform(String, key => [key]),
        ]).description('可用的 saucenao api key 列表。'),
        maxTrials: koishi_1.Schema.natural().description('最大尝试访问次数。').default(3),
        lowSimilarity: koishi_1.Schema.number().description('相似度较低的认定标准（百分比）。当 saucenao 给出的相似度低于这个值时，将不会显示 saucenao 本身的搜索结果 (但是 ascii2d 的结果会显示)。').default(40),
        highSimilarity: koishi_1.Schema.number().description('相似度较高的认定标准（百分比）。当 saucenao 给出的相似度高于这个值时，将不会使用 ascii2d 再次搜索。').default(60),
    }).description('搜索设置');
})(saucenao || (saucenao = {}));
exports.default = saucenao;
