"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkHost = exports.getShareText = exports.Config = exports.SearchConfig = exports.OutputConfig = exports.getLink = void 0;
const koishi_1 = require("koishi");
function getLink(url) {
    const pidSearch = /pixiv.+illust_id=(\d+)/.exec(url);
    if (pidSearch)
        return 'https://www.pixiv.net/i/' + pidSearch[1];
    const uidSearch = /pixiv.+member\.php\?id=(\d+)/.exec(url);
    if (uidSearch)
        return 'https://www.pixiv.net/u/' + uidSearch[1];
    return url;
}
exports.getLink = getLink;
exports.OutputConfig = koishi_1.Schema.object({
    thumbnail: koishi_1.Schema.boolean().default(true).description('是否在搜索结果中显示缩略图。'),
}).description('输出设置');
exports.SearchConfig = koishi_1.Schema.object({
    saucenaoApiKey: koishi_1.Schema.union([
        koishi_1.Schema.array(String),
        koishi_1.Schema.transform(String, key => [key]),
    ]).description('可用的 SauceNAO API key 列表。'),
    maxTrials: koishi_1.Schema.natural().description('最大尝试访问次数。').default(3),
    lowSimilarity: koishi_1.Schema.number().description('相似度较低的认定标准 (百分比)。当 SauceNAO 给出的相似度低于这个值时，将不会显示 SauceNAO 本身的搜索结果 (但是 ascii2d 的结果会显示)。').default(40),
    highSimilarity: koishi_1.Schema.number().description('相似度较高的认定标准 (百分比)。当 SauceNAO 给出的相似度高于这个值时，将不会使用 ascii2d 再次搜索。').default(60),
}).description('搜索设置');
exports.Config = koishi_1.Schema.intersect([
    exports.SearchConfig,
    koishi_1.Schema.object({
        output: exports.OutputConfig,
    }),
    koishi_1.Quester.createConfig(),
]);
function getShareText({ imageUrl, title, thumbnail, authorUrl, source }, config) {
    const output = [title];
    if (config.thumbnail)
        output.push(koishi_1.segment.image(thumbnail).toString());
    if (imageUrl)
        output.push(`链接：${getLink(imageUrl)}`);
    if (authorUrl)
        output.push(`作者：${getLink(authorUrl)}`);
    if (source)
        output.push(`来源：${getLink(source)}`);
    return output.join('\n');
}
exports.getShareText = getShareText;
function checkHost(source, name) {
    return source && source.includes(name);
}
exports.checkHost = checkHost;
